/*
 *  Copyright (c) 2005, FlexiGuided GmbH, Berlin, Germany
 *  Author: Jan Behrens <jan.behrens@flexiguided.de>
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the FlexiGuided GmbH nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 *  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 *  File name:    errhdl.c
 *  Version:      0.2
 *  Last changed: 2005-12-13
 *
 *  Dependencies: errhdl.h
 *
 *  Description:
 *  C file for error handling library, defining a global state variable to
 *  be used in non-multithreading cases and implementing functions needed by
 *  macros.
 *
 *  WARNING: Using compilers for comiling either the error handling library
 *  or programs using the library, which optimize the generated code, is
 *  dangerous!
 *  USE OPTIMIZED COMPILATION ONLY IF YOU KNOW WHAT YOU ARE DOING!
 *  For disabling optimization for the GNU C compiler search for the -O0
 *  switch in the manpage of the GNU C compiler.
 *
 */


#include "errhdl.h"
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

errhdl_state_t errhdl_state_global = { 0, };
errhdl_state_get_fptr errhdl_state_get = NULL;

void errhdl_error(char *class, char *msgfmt, ...) {
	va_list ap;
	if (!class) return;
	errhdl_state->class = class;
	va_start(ap, msgfmt);
	vsnprintf(errhdl_state->message, ERRHDL_MSGLEN+1, msgfmt, ap);
	va_end(ap);
	if (errhdl_state->envptr) longjmp(*errhdl_state->envptr, 1);
	else {
		if (errhdl_state->panic) {
			errhdl_state->panic(class, errhdl_state->message);
		} else {
			fprintf(stderr,
				"Unhandled error of class '%s': %s\n",
				class, errhdl_state->message);
			abort();
		}
	}
}

bool errhdl_errorofclass(char *subclass, char *superclass) {
	size_t i;
	if (!subclass) return false;
	for (i=0; ; i++) {
		if (superclass[i]) {
			if (subclass[i] != superclass[i]) return false;
		} else {
			if (subclass[i]=='.' || subclass[i]==0) return true;
			else return false;
		}
	}
}


