/*
 *  Copyright (c) 2005, FlexiGuided GmbH, Berlin, Germany
 *  Author: Jan Behrens <jan.behrens@flexiguided.de>
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the FlexiGuided GmbH nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 *  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 *  File name:    errhdl_thread.c
 *  Version:      0.2
 *  Last changed: 2005-12-13
 *
 *  Dependencies: errhdl.h errhdl_pthread.h
 *
 *  Description:
 *  C file for POSIX thread extensions to the error handling library.
 *
 *  WARNING: Using compilers for comiling either the error handling library
 *  or programs using the library, which optimize the generated code, is
 *  dangerous!
 *  USE OPTIMIZED COMPILATION ONLY IF YOU KNOW WHAT YOU ARE DOING!
 *  For disabling optimization for the GNU C compiler search for the -O0
 *  switch in the manpage of the GNU C compiler.
 *
 */


#include "errhdl_pthread.h"
#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

static pthread_key_t errhdl_state_key;

static void errhdl_panic_pthread() {
	fprintf(stderr,
		"An error occured while initializing error handling "
		"facilities for POSIX thread: %s\n", strerror(errno));
	abort();
}

static errhdl_state_t *errhdl_state_get_pthread() {
	return pthread_getspecific(errhdl_state_key);
}

static void errhdl_state_create() {
	if ((errno = pthread_key_create(&errhdl_state_key, free)))
		errhdl_panic_pthread();
	errhdl_state_get = errhdl_state_get_pthread;
}

void errhdl_state_init_pthread() {
	errhdl_state_t *state;
	static pthread_once_t keycreated[1] = { PTHREAD_ONCE_INIT };
	if ((errno = pthread_once(keycreated, errhdl_state_create)))
		errhdl_panic_pthread();
	state = calloc(1, sizeof(errhdl_state_t));
	if (!state) { errno = ENOMEM; errhdl_panic_pthread(); }
	if ((errno = pthread_setspecific(errhdl_state_key, state)))
		errhdl_panic_pthread();
}


