/*
 *  Copyright (c) 2005, FlexiGuided GmbH, Berlin, Germany
 *  Author: Jan Behrens <jan.behrens@flexiguided.de>
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the FlexiGuided GmbH nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 *  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 

/*
 *  File name:    regex_errhdl.c
 *  Version:      [not yet]
 *  Last changed: 2005-12-13
 *
 *  Dependencies: errhdl.h regex_errhdl.h
 *
 *  Description:
 *  C file implementing the wrapper library for regular expression functions
 *  in libc supporting exception handling.
 *
 *  NOTICE: Before using compiler optimization, please read the warning in
 *  the errhdl.h file.
 *
 */


#include "regex_errhdl.h"

void regcomp_errhdl(regex_t *preg, char *pattern, int cflags) {
	switch (regcomp(preg, pattern, cflags)) {
		case 0: return;
		case REG_ESPACE: MemoryError();
		default: Error("regex.compile",
			"Could not compile regular expression.");
	}
}

bool regexec_errhdl(regex_t *preg, char *string,
		size_t nmatch, regmatch_t *pmatch, int eflags) {
	switch (regexec(preg, string, nmatch, pmatch, eflags)) {
		case 0: return true;
		case REG_NOMATCH: return false;
		case REG_ESPACE: MemoryError();
		default: Error("regex.exec",
			"Could not execute regular expression.");
	}
	return false;
}


